
//////////////////////////////////////////////////////////////////////
//  si5351 VFO program ver.1.0
//    Copyright(C)2016.JA2GQP.All rights reserved.
//
//                                Arduino IDE 2.0.0 Compiled
//
//                                                2023/8/18 prepraveno pod Klopik za 7Mhz (LZ1TKA)
//                                                  JA2GQP
//--------------------------------------------------------------------
//  Function
//    1.STEP(50k,5k,500,50)
//    2.Memory Channel ch0 - ch9(10ch)
//    3.Protection Operation At The Time Of Transmission
//--------------------------------------------------------------------
// Library
//  Rotary encoeder  http://www.buxtronix.net/2011/10/rotary-encoders-done-properly.html
//  si5351Arduino    https://github.com/etherkit/Si5351Arduino
//  LCD5110_Basic    http://www.rinkydinkelectronics.com/library.php?id=44
//////////////////////////////////////////////////////////////////////

#include <Rotary.h>
#include <si5351.h>
#include <Wire.h>
#include <LCD5110_Basic.h>
#include <EEPROM.h>
//#include <Adafruit_GFX.h>
//#include <Adafruit_PCD8544.h>


//----------  Define Constant Value   -----------------------

////////////////////////////////
// I/O assign
////////////////////////////////

const byte ENC_A = 2;          // Encorder A
const byte ENC_B = 3;          // Encoeder B
const byte SW_STEP = 4;        // STEP SW
const byte SW_RIT = 5;         // RIT SW
const byte SW_CH = 6;          // CH SW
const byte SW_TX = 15;         // TX SW
const byte But_RIT_Plus = 0;   // buton za funkciq Rit, izmenq chestotata na heterodina CLk2
const byte But_RIT_Minus = 1;  // buton za funkciq Rit, izmenq chestotata na heterodina CLk2
//LCD5110 myGLCD(8, 9, 10, 11, 12)  // SCK,MOSI,DC,RST,CS


const byte AD_IN = A0;  // analog input for left channel

////////////////////////////////
// default value
////////////////////////////////

const long DEF_FRQ = 7000000L;  // Default Vfo(7.00MHz)
const long DEF_STP = 500L;     // Init STEP(500Hz)
////////////////////////////////
// Limited range
////////////////////////////////

const long LW_FRQ = 7000000L;  // Frequency Lower Limit
const long HI_FRQ = 7200000L;  //           Upper Limit

const long LW_RIT = -1000L;  // RIT Lower Limit
const long HI_RIT = 1000L;   // RIT Upper Limit

const long LW_VFO80 = 3500000L;   // 3.5MHz Lower
const long MI_VFO80 = 3535000L;   //        Middle
const long HI_VFO80 = 3575000L;   //        Upper
const long LW_VFO40 = 7000000L;   // 7MHz   Lower
const long MI_VFO40 = 7045000L;   //        Middle
const long HI_VFO40 = 7200000L;   //        Upper
const long LW_VFO20 = 14000000L;  // 14MHz  Lower
const long MI_VFO20 = 14100000L;  //        Middle
const long HI_VFO20 = 14350000L;  //        Upper
const long LW_VFO15 = 21000000L;  // 21MHz  Lower
const long MI_VFO15 = 21150000L;  //        Middle
const long HI_VFO15 = 21450000L;  //        Upper
const long LW_VFO10 = 28000000L;  // 28MHz  Lower
const long MI_VFO10 = 28200000L;  //        Middle
const long HI_VFO10 = 29000000L;  //        Upper

////////////////////////////////
// etc
////////////////////////////////

const byte Max_Chn = 3;   // Broq na kanalite za pamet,maksimalno 0 do 9
const byte Int_End = 73;  // Initial end code

//----------  EEPROM Memory Address   -----------------------

const byte Frq_Eep = 0x00;  // Frequency(4byte*10)
const byte Stp_Eep = 0x30;  // STEP(4byte*10)
const byte Chn_Eep = 0x60;  // Channel(1byte*1)
const byte Eep_Int = 0x6e;  // Eep Init(1byte*1)

//----------  LCD NOKIA 5110 definition --------------------

LCD5110 myGLCD(8, 9, 10, 11, 12);  // SCK,MOSI,DC,RST,CS
extern uint8_t SmallFont[];
extern uint8_t MediumNumbers[];
extern uint8_t BigNumbers[];
//----------  si5351 definition -----------------------------

Si5351 si5351;

//----------  Encorder Pin definition  ----------------------

Rotary r = Rotary(ENC_A, ENC_B);

//----------  Memory Assign  --------------------------------

volatile long LSB = 8988610L;  // 8989000L   IF +1110 (0,3335*1500 = 500.25)
volatile long USB = 8986390L;  // 8986900L   IF - 1110
volatile long CW =  8987500L;   //8988100L   IF + 600
volatile long bfo = 8987500L;  // 10701500  start in LSB   8989000L   IF +1500 8987500L; (8988600 = + 1100  ???)
volatile long IF = 5330112L;   // CW 5.330112MHz  8.987500      Start na Mejdinnata ot 7 Megaherza za kvarc 8.987500

volatile long Vfo_Dat = DEF_FRQ;  // Default Frequency
volatile long Vfo_Datb;           // Vfo data(old)
volatile long Lng_Wk1;            // Long Work1
volatile long Lng_Wk2;            // Long Work2
volatile long Dds_Dat;
volatile long Vfo_Dat_Mej = 0;  // Korekciq za Mejdinna chestota na 5 gigaherza
volatile long Mej = 5329611.75L;
volatile long Obh = 0;  // Atart na Obhvata za izchislqvane na mejdinnata
volatile long Obh_a = 0;
volatile long Cor_vfo_az = 1700;  // koregtira chestotata za  LSB USB,SW , izcislenieto e sprqmo stoinost 0,3335 za mejdinata na 5mhz. Moje da se polzva za korekciq na chestotata na Mdejdinnata pri nujda.
volatile long Cor_dis_az = 0;     //Koregira chestotata izlachvana kam si5351 sprqmo  LSB USB,SW
String tbfo = "";

volatile long Rit_Dat = 0;  // RIT Data
volatile long Rit_Datb = 0;
volatile long Enc_Stp = 1000;  // STEP

byte Flg_Tx = 0;    // TX Flag
byte Flg_Rit = 0;   // RIT Flag
byte Flg_Spl;       // SPLIT Flag
byte Flg_Over;      // Over Flag
byte Byt_Chn = 0;   // Channel SW
byte Byt_Chnb = 0;  // Channel SW Old
byte Ch_dis = 0;    // Chisti displeya
byte Flg_but_Plus = 0; // Butoni za CLK0
byte Flg_but_Minus = 0; // Butoni za CLK0
byte FCLV_LOOP1;
byte FCV_A;
byte FCV_B;
//----------  Initialization  Program  ----------------------

void setup() {
  myGLCD.InitLCD();  // nokia5110 Init
  Wire.begin();

  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0);        //initialize the Si5351
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);  // Set PLL
                                                  // Set CLK0 Frequency(VFO)
  pinMode(SW_STEP, INPUT_PULLUP);
  pinMode(SW_RIT, INPUT_PULLUP);
  pinMode(SW_CH, INPUT_PULLUP);
  pinMode(SW_TX, INPUT_PULLUP);
  pinMode(But_RIT_Plus, INPUT_PULLUP);
  pinMode(But_RIT_Minus, INPUT_PULLUP);

  PCICR |= (1 << PCIE2);  // Enable pin change interrupt for the encoder
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();  // INT Enable
  Rit_Dat = 0;
  Flg_Rit = 0;

  if (EEPROM.read(Eep_Int) != Int_End) {  // Eep initialaz
    delay(10);
    Fnc_Eep_Int();
  }

  Byt_Chn = EEPROM.read(Chn_Eep);  // Eep Channel Read
  Byt_Chnb = Byt_Chn;
  Fnc_Eep_Rd();
  Fnc_Lcd();  // Display LCD
  Fnc_Step_Disp();
}

//----------  Main program  ---------------------------------

void loop() {
  if (Flg_Tx == 0) {
    Fnc_Smeter();

    if (digitalRead(SW_STEP) == LOW)  // STEP Sw On?    BUTONI
      Fnc_Stp();
    if ((digitalRead(SW_RIT) == LOW))  // RIT SW On?    BUTONI
      Fnc_Rit();
    if ((digitalRead(SW_CH) == LOW))  // CH SW On?    BUTONI
      Fnc_Chsw();
    if (digitalRead(But_RIT_Plus) == LOW)  // Buton za CLK0
      //Fnc_But_Plus();
    if (digitalRead(But_RIT_Minus) == LOW)  // Buton za CLK0
      //Fnc_Bu_Minus();

    if (Byt_Chnb != Byt_Chn) {  // CH SW OLD != NEW?
      Fnc_Eep_Wt(Byt_Chnb);
      Byt_Chnb = Byt_Chn;
      Fnc_Eep_Rd();
    }
  }

  if (digitalRead(SW_TX) == LOW)  // Tx On?
    Flg_Tx = 1;
  else
    Flg_Tx = 0;

  Fnc_Step_Disp();
  Fnc_Lcd();
  //-----------  Izchislenmie mejdu Mejdinnata i pokazaniq na displeya--------------
  //-----------  Vzima pokazanie na displeq i gi prevrashta v stoinost za razlichna mejdina chestota---------
  if (Obh == Obh_a) {                      // Sravnqva vhodqstata chestota s izhodqstata,ako e sashtata otkazva promeni za VFO
  } else {
    Vfo_Dat_Mej = 0;                       //nulira
    Mej = 5330112L;                        //zadava postoqna stoinost na mejdinata(5330112) pri izpisvane na displeq ravnqvashta se na 7 Mhz za FILTRI s kvarcove na 8987500 Mhz
    Vfo_Dat_Mej = Obh - DEF_FRQ;           // izchislqva s kolko herza se e promenila chestotata izpisana na  displeya
    Vfo_Dat_Mej = Vfo_Dat_Mej * 0.3335;    // umnojava herzovete sprqmo stoinosta na Mejdinnata ot 5 megaherza ,stoinosta e za 1 Herz!
    Cor_dis_az = Cor_vfo_az * 0.3335;      // umnojava herzovete sprqmo stoinosta na Mejdinnata ot 5 megaherza ,stoinosta e za 1 Herz,dobavq za LSB ili USB
    Mej = Mej + Vfo_Dat_Mej - Cor_dis_az;  // Sabira se minamalnata mejdina na 7000000 s poluchenata stoinost v hercove.
    Obh_a = Obh;
  }

  //---------------------------------------------------------------------------
  if ((Flg_Tx == 0) && (Flg_Rit == 1))
    si5351.set_freq(((Mej - Rit_Dat) * SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK0);
  if ((Flg_Tx == 1) || (Flg_Rit == 0))
    si5351.set_freq(((Mej)*SI5351_FREQ_MULT), SI5351_PLL_FIXED, SI5351_CLK0);  // Podava kam si5351 izbranata mejdinna chestota prez CLK0

  Fnc_Band(Vfo_Dat);
  if (Flg_Over == 1)                  // Over?
    myGLCD.print("over", CENTER, 0);  // Display(over)   Izvan obhvata,ne se polzva v skecha
  else
    //myGLCD.clrRow(2);                           // Clear(over)  Chisti centralniq red 16-ta poziciq ot gore nadolu

    if ((Flg_Over == 1) && (Flg_Tx == 1))
      si5351.output_enable(SI5351_CLK0, 0);  // VFO disable
    else
      si5351.output_enable(SI5351_CLK0, 1);  // VFO enable
      si5351.set_freq((bfo * SI5351_FREQ_MULT), 0, SI5351_CLK2);  // BFO! IZHOD NA PLATKATA CLK2 (Podava kam si5351 izbranata  chestota prez CLK2)
}

//----------  Function S-Meter Display  -----------------------

void Fnc_Smeter() {
  int ad1 = 0;
  int ad2 = 0;

  for (int i = 0; i < 6; i++) {
    ad1 = map(sqrt(analogRead(AD_IN) * 20), 0, 143, 0, 140);  // 20-daljina
    ad2 = ad1 + ad2;
  }
  ad2 = ad2 / 5;
  int s_dat = map(ad2, 0, 140, 0, 14);

  for (int i = 0; i < s_dat; i++)
    myGLCD.print("\\", (i * 6), 40);  // "\\"-grafika na skalata
    myGLCD.clrRow(5, (s_dat * 6));

  //----Izpisvane na displeya----

  if (Flg_Rit == 1) {
  } else {
    myGLCD.print("   ", LEFT, 32);      // Chisti nadpisa R+0 ot displeq pri povtorno natiskane na butona RIT
    //myGLCD.printNumI(bfo, LEFT, 32);  // Izpisva  mejdinnata ili BFO nad S metara,pokazva kakva e v momenta,premesteno v menu PARAMETRI
    Ch_dis == 1;                      // Ako e imalo pokazanie na mejdinata varhu diapleq go markira
  }

  //----Pokazaniq na S Metara---

  if (s_dat < 1)
    myGLCD.print(" S0   ", RIGHT, 32);
  else if ((s_dat <= 1) && (s_dat < 2))
    myGLCD.print(" S1   ", RIGHT, 32);
  else if ((s_dat <= 2) && (s_dat < 4))
    myGLCD.print(" S3   ", RIGHT, 32);
  else if ((s_dat <= 4) && (s_dat < 5))
    myGLCD.print(" S5   ", RIGHT, 32);
  else if ((s_dat <= 5) && (s_dat < 7))
    myGLCD.print(" S7   ", RIGHT, 32);
  else if ((s_dat <= 7) && (s_dat < 9))
    myGLCD.print(" S9   ", RIGHT, 32);
  else if ((s_dat <= 9) && (s_dat < 12))
    myGLCD.print(" S9+10", RIGHT, 32);
  else if ((s_dat <= 12) && (s_dat < 13))
    myGLCD.print(" S9+20", RIGHT, 32);
  else if (s_dat >= 13)
    myGLCD.print(" S9+40", RIGHT, 32);
}

//----------  Encorder procedure(INT)  ---------------

ISR(PCINT2_vect) {
  unsigned char result = r.process();

  if (Flg_Tx == 0) {
    if (result) {
      if (result == DIR_CW) {
        Lng_Wk1 = Vfo_Dat + Enc_Stp;
        Lng_Wk2 = Rit_Dat + Enc_Stp;
      } else {
        Lng_Wk1 = Vfo_Dat - Enc_Stp;
        Lng_Wk2 = Rit_Dat - Enc_Stp;
      }

      if (Flg_Rit == 1)
        Rit_Dat = Lng_Wk2;
      else {
        Vfo_Dat = Lng_Wk1;
        Rit_Dat = 0;
      }

      Vfo_Dat = constrain(Vfo_Dat, LW_FRQ, HI_FRQ);  // VFO range check
      Rit_Dat = constrain(Rit_Dat, LW_RIT, HI_RIT);  // RIT range check
    }
  }
}

//----------  Function Encorder STEP  -----------------------

void Fnc_Stp() {
  if (Enc_Stp == 50)    // Step = 50Hz ?
    Enc_Stp = 5000000;  //   Yes,1Mhz set
  else
    Enc_Stp = Enc_Stp / 10;  // Step down 1 digit

  Fnc_Step_Disp();
  while (digitalRead(SW_STEP) == LOW)
    ;
}
//----------  Function STEP Display  ------------------------

void Fnc_Step_Disp() {
  switch (Enc_Stp) {
    case 50:
      myGLCD.print("  50 Hz", RIGHT, 24);
      myGLCD.invertText(false);  //   (true)  - (false) Smena fona na teksta na  bql
      break;
    case 500:
      myGLCD.invertText(true);  //   (true)  - (false) Smena fona na teksta na cheren
      myGLCD.print("500 Hz", RIGHT, 24);
      break;
    case 5000:
      myGLCD.invertText(true);  //   (true)  - (false)  Smena fona na teksta na cheren
      myGLCD.print(" 5 Khz", RIGHT, 24);
      break;
    case 50000:
      myGLCD.invertText(true);  //   (true)  - (false) Smena fona na teksta na cheren
      myGLCD.print(" 50 Khz", RIGHT, 24);
      break;
      //case 500000:
      //myGLCD.print(" 500k", RIGHT, 24);
      //break;
      //case 5000000:
      //myGLCD.print("  5M", RIGHT, 24);
      //break;
      //default:
      //myGLCD.print("  5k", RIGHT, 24);
      //Enc_Stp = 5000;
      //break;
  }
}

//----------  Function Save EEPROM 4byte  ---------

void Fnc_Eep_Sav4(long value, int address) {
  address += 3;
  for (int i = 0; i < 4; i++) {
    byte toSave = value & 0xFF;
    if (EEPROM.read(address) != toSave) {
      EEPROM.write(address, toSave);
    }
    value = value >> 8;
    address--;
  }
}

//----------  Function Load EEPROM 4byte  ---------

long Fnc_Eep_Lod4(int address) {
  long value = 0;

  for (int i = 0; i < 4; i++) {
    value = value | EEPROM.read(address);
    if (i < 3) {
      value = value << 8;
      address++;
    }
  }
  return value;

   
  
}
//---------- LCD Display --------------------------

void Fnc_Lcd() { 
  char s[6] = { '\0' };
  Obh = Vfo_Dat;
  Fnc_Fdsp(Vfo_Dat);  //Otgovarq za izpisvaneto na chestotata na displeya OTGORE! CHESTOTA NA RABOTA
  myGLCD.setFont(SmallFont);
  //--- Nadpis za zapameten kanal
  if (Flg_Tx == 0) {
    myGLCD.printNumI(Byt_Chn, 23, 24);  // Otgovarq za izpisvaneto na kanala za pameta         Byt_Chn myGLCD.printNumI(Byt_Chn,RIGHT, 0);
  } else {
  }
  myGLCD.print(tbfo, LEFT, 24);  // Mode  IZPISVA REDA V LQVO DOLU -lsb/usb/cw
  myGLCD.print("ch", 29, 24);    // izpisva "ch" za nomera na pameta na kanala
  //---
  if (Flg_Rit == 1) {
    myGLCD.print("R", 0, 32);
    if (Rit_Dat != Rit_Datb) {
      myGLCD.print("      ", 6, 32);
      Rit_Datb = Rit_Dat;
    }
    sprintf(s, "%+d", Rit_Dat);
    myGLCD.print(s, 6, 32);
  }

  if (Flg_Tx == 0)
    myGLCD.print("    ", CENTER, 22);  //Chisti nadpisa !!TX!!
  else
    myGLCD.print("!!TX!!", RIGHT, 32);  // Izpisva !!TX!! na predavane

 //  --------REKLAMA---- LZ1TKA

   if (FCV_A == 0)   
    {
          myGLCD.clrScr();                               // Chisti displeya ot predishni nadpisi
          myGLCD.print("LZ1TKA 73!", CENTER, 0);            // REKLAMA
          myGLCD.print("v 2.0", CENTER, 16);
          myGLCD.print("JA2GQP", CENTER, 30);
          myGLCD.print("Band 40m", CENTER, 40); 
          delay (2000);                                  // zadarja izpisvaneto na REKLAMATA 2000 (100 za TEST v Prooteusa)
          myGLCD.clrScr();                             // Chisti displeya ot predishni nadpisi
          FCV_A = 1;   
          } else {   
          }
        
//--------Izmenq chestotata na CLK2---- LZ1TKA

   
   if (Flg_but_Plus == 1 | Flg_but_Minus == 1)  // Ako e natisnat edin ot 2 butona za VFO vliza vremenno v  LOOP
    {
         myGLCD.clrScr(); // Chisti displeya ot predishni nadpisi

      for (FCLV_LOOP1=0; FCLV_LOOP1<10; FCLV_LOOP1++)  // lupi 10 pati predi
        {
          //-------Menu PARAMETRI----
         myGLCD.print("PARAMETRI", CENTER, 0);                       // Izpisva nadpisa PARAMETR
         myGLCD.printNumI(bfo, 0, 12);                               //Izpisva BFO
         myGLCD.print("=  BFO", RIGHT, 12);                         
         myGLCD.printNumI(Mej, 0, 26);                              //Izpisva Mejdinnata
         myGLCD.print("=  VFO", RIGHT, 26);
         myGLCD.printNumI(Vfo_Dat, 0, 36);                          //Izpisva Chestotata na Displeya
         myGLCD.print("=  Mhz", RIGHT, 36);

         delay (50);                                                // zadarja izpisvaneto na parametrite za BFO na displeq
        }
        
        myGLCD.print("                    ", RIGHT, 10);            // CHISTI REDOVETE  sled BFO
        myGLCD.print("                    ", RIGHT, 26);            // CHISTI REDOVETE NA DISPLEYA sled Mejdinnata
        myGLCD.print("                    ", RIGHT, 36);            // CHISTI REDOVETE NA DISPLEYA sled Chestotata
    // } else {

    }
    if (digitalRead(But_RIT_Plus) == LOW)  //  Chete butona za VFO
    Flg_but_Plus = 1;
    else
    Flg_but_Plus = 0;
   if (digitalRead(But_RIT_Minus) == LOW)  //  Chete butona za VFO
    Flg_but_Minus = 1;
    else
    Flg_but_Minus = 0;
   if (Flg_but_Plus == 1) {  // Uvelichava LSB s 10 edinici pri natisnat buton
     LSB = LSB + 10;  
     } else { 
     }
   if (Flg_but_Minus == 1) {  // Namalqvaa LSB s 10 edinici pri natisnat buton
     LSB = LSB - 10; 
     } else {
     }
  
}

//----------  Function Frequency Display  ---------

void Fnc_Fdsp(long f_disp) {
  long f1 = f_disp / 1000L;
  long f2 = f_disp % 1000L;
  char s1[6] = { '\0' };
  char s2[4] = { '\0' };

  if (f_disp != Vfo_Datb) { 
    myGLCD.clrRow(0);
    myGLCD.clrRow(1);
    Vfo_Datb = Vfo_Dat;
  }
  myGLCD.setFont(MediumNumbers);
  myGLCD.printNumI(f1, LEFT, 0);  //Izpisva chestotata s golemi cifri v LQVO
  sprintf(s2, "%03d", f2);
  myGLCD.print(s2, RIGHT, 0);  //   Izpisva chestotata s golemi cifri v DQSNO
}


//----------  Function Rit  ---------

void Fnc_Rit() {
  char s[6] = { '\0' };

  if (Flg_Rit == 0) {
    Rit_Dat = 0;
    Fnc_Eep_Wt(Byt_Chn);
    myGLCD.print("R", 0, 32);
    sprintf(s, "%+d", Rit_Dat);
    myGLCD.print(s, 6, 32);
    Flg_Rit = 1;

    if (Ch_dis == 1) {
    } else {
      myGLCD.print("        ", 0, 32);  // Chisti reda na displeya ot pokazanieto za mejdinna
    }
  } else {
    Flg_Rit = 0;
    //myGLCD.print("    ",CENTER,16);   // Predishno izpisvane za izchistvane
  }

  while (digitalRead(SW_RIT) == LOW)
    ;
}

//----------  Function CH SW Check  ---------

void Fnc_Chsw() {
  byte cnt = 0;

  Byt_Chn++;

  while (digitalRead(SW_CH) == LOW) {
    delay(500);
    cnt++;
    if (6 <= cnt) {                    // Eep Initial start(3sec)?
      Fnc_Eep_Int();                   // Initialization
      Byt_Chn = EEPROM.read(Chn_Eep);  // Channel Read
      Byt_Chnb = Byt_Chn;
      Fnc_Eep_Rd();  // EEPROM Read
      Fnc_Fdsp(Vfo_Dat);
      myGLCD.print("default", CENTER, 16);
    }
  }
}

//----------  Function Band  -------------------------------

void Fnc_Band(long vfo) {
  if ((vfo >= LW_VFO80) && (vfo < MI_VFO80)) {  // 3.5MHz
    Flg_Over = 0;
    tbfo = "CW ";
    //bfo = CW;
    Cor_vfo_az = 200;
  } else if ((vfo >= MI_VFO80) && (vfo <= HI_VFO80)) {
    Flg_Over = 0;
    tbfo = "LSB";
    //bfo = LSB;
    Cor_vfo_az = Cor_vfo_az = 1700;
  }

  else if ((vfo >= LW_VFO40) && (vfo < MI_VFO40)) {  // 7MHz ZadavA INTERVAL NA CHASTOTA ZA RABOTA
    Flg_Over = 0;
    tbfo = "CW ";
    bfo = CW;
    Cor_vfo_az = 200;
  } else if ((vfo >= MI_VFO40) && (vfo <= HI_VFO40)) {
    Flg_Over = 0;
    tbfo = "LSB";
    bfo = LSB;
    Cor_vfo_az = 1700;
  }

  else if ((vfo >= LW_VFO20) && (vfo < MI_VFO20)) {  // 14MHz
    Flg_Over = 0;
    tbfo = "CW ";
    //bfo = CW;
    Cor_vfo_az = 200;
  } else if ((vfo >= MI_VFO20) && (vfo <= HI_VFO20)) {
    Flg_Over = 0;
    tbfo = "USB";
    //bfo = USB;
    //Cor_vfo_az = Cor_vfo_az - 1700;
  }

  else if ((vfo >= LW_VFO15) && (vfo < MI_VFO15)) {  // 21MHz
    Flg_Over = 0;
    tbfo = "CW ";
    //bfo = CW;
    Cor_vfo_az = 200;
  } else if ((vfo >= MI_VFO15) && (vfo <= HI_VFO15)) {
    Flg_Over = 0;
    tbfo = "USB";
    //bfo = USB;
    //Cor_vfo_az = -1700;
  }

  else if ((vfo >= LW_VFO10) && (vfo < MI_VFO10)) {  // 28MHz
    Flg_Over = 0;
    tbfo = "CW ";
    //bfo = CW;
    Cor_vfo_az = 200;
  } else if ((vfo >= MI_VFO10) && (vfo <= HI_VFO10)) {
    Flg_Over = 0;
    tbfo = "USB";
    //bfo = USB;
    //Cor_vfo_az = -1700;
  }

  else if (Vfo_Dat < 10000000L) {
    //bfo = LSB;
    tbfo = "LSB";
    Flg_Over = 1;
    Cor_vfo_az = 1700;
  } else {
    //bfo = USB;
    tbfo = "USB";
    Flg_Over = 1;
    //Cor_vfo_az = -1700;
  }
}

//---------- Function Eeprom Initialization -----------------

void Fnc_Eep_Int() {
  int i;

  for (i = 0; i < 112; i++)  // 0 clear(112byte)
    EEPROM.write(i, 0);

  for (i = 0; i < Max_Chn; i++) {
    Fnc_Eep_Sav4(DEF_FRQ, Frq_Eep + i * 4);  // Frequency(7.10MHz)
    Fnc_Eep_Sav4(DEF_STP, Stp_Eep + i * 4);  // Step(1kHz)
  }

  EEPROM.write(Eep_Int, Int_End);  // Init end set(73)
}

//----------  Function EEPROM Read  ---------

void Fnc_Eep_Rd() {
  if ((0 <= Byt_Chn) && (Byt_Chn < Max_Chn))
    Vfo_Dat = Fnc_Eep_Lod4(Frq_Eep + Byt_Chn * 4);
  else {
    Vfo_Dat = Fnc_Eep_Lod4(Frq_Eep + 0 * 4);
    Byt_Chn = 0;
  }

  if ((0 <= Byt_Chn) && (Byt_Chn < Max_Chn))
    Enc_Stp = Fnc_Eep_Lod4(Stp_Eep + Byt_Chn * 4);
  else
    Enc_Stp = Fnc_Eep_Lod4(Stp_Eep + 0 * 4);
}

//----------  Function EEPROM Write  ---------

void Fnc_Eep_Wt(byte chn) {
  if ((0 <= chn) && (chn < Max_Chn)) {
    Fnc_Eep_Sav4(Vfo_Dat, Frq_Eep + chn * 4);
    Fnc_Eep_Sav4(Enc_Stp, Stp_Eep + chn * 4);
  }

  EEPROM.write(Chn_Eep, chn);
}
